/*
 * Copyright (c) 1993  Kenji Arisawa. All right reserved.
 * License to freely use and distribute this software is hereby granted 
 * by the author, subject to the condition that this copyright notice 
 * remains intact.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY WARRANTIES.
 *
 * You should read comment in slip.c
 * -Kenar-
 */

#include "slip.h"

int vflag = NO;

void usage(name)
char *name; /* argv[0] */
{
	fprintf(stderr, "usage: %s [-v] sa=SA da=DA sm=SM [st=ST] [rt]\n",name);
	fputs("   SA: slip address\n",stderr);
	fputs("   DA: slip destination address\n",stderr);
	fputs("   SM: slip mask\n",stderr);
	fputs("   ST: slip type (currently unused)\n",stderr);
	fputs("   rt: set route if default route non-existent\n",stderr);
	exit(1);
}

/*
 * show: show memory of size n in printable format. ( tip2 )
 */
void show(cp, n)
char *cp;
int n;
{
	register int i;
	char c;

	for(i = 0; i < n; i++) {
		c = cp[i]&0177;
		if(isprint(c))
			fputc(c, stderr);
		else
			fprintf(stderr, "\\%03o", c);
	}
}

static sigfunc_t naptimeout()
{
	/* make SIGALRM no effect */
	/* return(0); */
}


/*
 * nap: sleep a little ( tip2 )
 * EXAMPLE
 * nap(20) sleeps 20ms
 * nap(0)  clears timer. ( 1996/01 tip+ )
 * -Kenar-
 */
void nap(ms)
int ms;
{
	struct itimerval itv;
	signal(SIGALRM, naptimeout);
	itv.it_interval.tv_sec = 0;
	itv.it_interval.tv_usec = 0;
	itv.it_value.tv_sec = 0;
	itv.it_value.tv_usec = ms*1000;
	setitimer(ITIMER_REAL, &itv, (struct itimerval *)0);
	/* The ITIMER_REAL timer decrements in real time.
	 * A SIGALRM signal is delivered when this timer expires.
	 * FROM man setitimer
	 * -Kenar- */
	if(ms > 0){
		sigpause(0);
		/* Sigpause always terminates by being interrupted,
		 * returning -1 with errno set to EINTR.
		 * FROM man sigpause
		 * Kenar say,
		 *   I will use SIGALRM for this purpose. This works well.
		 * -Kenar- */
		nap(0);
	}
}


main(argc,argv)
char *argv[];
{
	char *data = NULL;
	int i,ch;
	char *slipname,*p,*sa,*da,*sm,*st;
	char buf[256];
	boolean rt = NO;
	extern int optind;
	
	slipname = argv[0];
	while ((ch = getopt(argc,argv,"hv")) != EOF){
		switch (ch){
		case 'h':
			usage(slipname);
		case 'v':
			vflag = YES;
			break;
		default:
			usage(slipname);
		}
	}
	i = optind;
	if(i < argc) data = argv[i];
	if(!data) usage(slipname);
	
	
	while(i < argc){
		strcpy(buf,argv[i]);
		/* DEBUG: puts(buf); */
		p = index(buf,'=');
		if(p){
			*p++ = '\0';
			if(equal(buf,"sa")) sa = makstr(p);
			else if(equal(buf,"da")) da = makstr(p);
			else if(equal(buf,"sm")) sm = makstr(p);
			else if(equal(buf,"st")) st = makstr(p);
			else if(equal(buf,"rt")){
				if(equal(p,"ON")) rt = YES;
				else if(equal(p,"OFF")) rt = NO;
			}
		}
		i++;
	}
	
	/* DEBUG:
	   puts(sa);
	   puts(da);
	   puts(sm);
	   puts(st);
	   if(rt) puts("NO");
	   else puts("YES");
	   return;
	 */
	
	/* we temporary neglect st */
	runslip(sa,da,sm,rt);
}
